@@ -7,11 +7,6 @@
   if (!exists(".tidyllm_rate_limit_env", envir = .GlobalEnv)) {
     .GlobalEnv$.tidyllm_rate_limit_env <- new.env(parent = emptyenv())
   }
-  #Initialize the parent environment for streaming backups on package load
-  if (!exists(".tidyllm_stream_env", envir = .GlobalEnv)) {
-    .GlobalEnv$.tidyllm_stream_env <- new.env(parent = emptyenv())
-  }
-
 }
 
 # enable usage of <S7_object>@name in package code
